@@ -7,6 +7,7 @@ import json
 
 import requests
 import urllib3
+from warnings import warn
 
 from .const import ERROR_CODES
 from .media_stream.session import HttpMediaSession
@@ -111,10 +112,9 @@ class Tapo:
                 return self.performRequest(requestData, True)
             else:
                 raise Exception(
-                    "Error: "
-                    + self.getErrorMessage(data["error_code"])
-                    + " Response:"
-                    + json.dumps(data)
+                    "Error: {}, Response: {}".format(
+                        self.getErrorMessage(data["error_code"]), json.dumps(data)
+                    )
                 )
 
     def getMediaSession(self):
@@ -289,6 +289,18 @@ class Tapo:
             {"method": "do", "motor": {"movestep": {"direction": str(angle)}}}
         )
 
+    def moveMotorClockWise(self):
+        return self.moveMotorStep(0)
+
+    def moveMotorCounterClockWise(self):
+        return self.moveMotorStep(180)
+
+    def moveMotorVertical(self):
+        return self.moveMotorStep(90)
+
+    def moveMotorHorizontal(self):
+        return self.moveMotorStep(270)
+
     def calibrateMotor(self):
         return self.performRequest({"method": "do", "motor": {"manual_cali": ""}})
 
@@ -305,13 +317,6 @@ class Tapo:
             }
         )
 
-    def setDayNightMode(self, inf_type):
-        if inf_type not in ["off", "on", "auto"]:
-            raise Exception("Invalid inf_type, can be off, on or auto")
-        return self.performRequest(
-            {"method": "set", "image": {"common": {"inf_type": inf_type}}}
-        )
-
     def getUserID(self):
         if not self.userID:
             self.userID = self.performRequest(
@@ -330,7 +335,7 @@ class Tapo:
         return self.userID
 
     def getRecordings(self, date):
-        return self.performRequest(
+        result = self.performRequest(
             {
                 "method": "multipleRequest",
                 "params": {
@@ -352,9 +357,13 @@ class Tapo:
                     ]
                 },
             }
-        )["result"]["responses"][0]["result"]["playback"]["search_video_results"]
+        )["result"]["responses"][0]["result"]
+        if "playback" not in result:
+            raise Exception("Video playback is not supported by this camera")
+        return result["playback"]["search_video_results"]
 
     def getCommonImage(self):
+        warn("Prefer to use a specific value getter", DeprecationWarning, stacklevel=2)
         return self.performRequest({"method": "get", "image": {"name": "common"}})
 
     def setMotionDetection(self, enabled, sensitivity=False):
@@ -407,7 +416,7 @@ class Tapo:
 
     def deletePreset(self, presetID):
         if not str(presetID) in self.presets:
-            raise Exception("Preset " + str(presetID) + " is not set in the app")
+            raise Exception("Preset {} is not set in the app".format(str(presetID)))
 
         self.performRequest(
             {"method": "do", "preset": {"remove_preset": {"id": [presetID]}}}
@@ -417,41 +426,85 @@ class Tapo:
 
     def setPreset(self, presetID):
         if not str(presetID) in self.presets:
-            raise Exception("Preset " + str(presetID) + " is not set in the app")
+            raise Exception("Preset {} is not set in the app".format(str(presetID)))
         return self.performRequest(
             {"method": "do", "preset": {"goto_preset": {"id": str(presetID)}}}
         )
 
-    def getLensDistortionCorrection(self):
+    # Switches
+
+    def __getImageSwitch(self, switch: str) -> str:
         data = self.performRequest({"method": "get", "image": {"name": ["switch"]}})
-        return data["image"]["switch"]["ldc"] == "on"
+        switches = data["image"]["switch"]
+        if switch not in switches:
+            raise Exception("Switch {} is not supported by this camera".format(switch))
+        return switches[switch]
+
+    def __setImageSwitch(self, switch: str, value: str):
+        return self.performRequest(
+            {"method": "set", "image": {"switch": {switch: value}},}
+        )
+
+    def getLensDistortionCorrection(self):
+        return self.__getImageSwitch("ldc") == "on"
 
     def setLensDistortionCorrection(self, enable):
-        return self.performRequest(
-            {"method": "set", "image": {"switch": {"ldc": "on" if enable else "off"}}}
-        )
+        return self.__setImageSwitch("ldc", "on" if enable else "off")
 
     def getImageFlipVertical(self):
-        data = self.performRequest({"method": "get", "image": {"name": ["switch"]}})
-        return data["image"]["switch"]["flip_type"] == "center"
+        return self.__getImageSwitch("flip_type") == "center"
 
     def setImageFlipVertical(self, enable):
+        return self.__setImageSwitch("flip_type", "center" if enable else "off")
+
+    def getForceWhitelampState(self) -> bool:
+        return self.__getImageSwitch("force_wtl_state") == "on"
+
+    def setForceWhitelampState(self, enable: bool):
+        return self.__setImageSwitch("force_wtl_state", "on" if enable else "off")
+
+    # Common
+
+    def __getImageCommon(self, field: str) -> str:
+        data = self.performRequest({"method": "get", "image": {"name": "common"}})
+        fields = data["image"]["common"]
+        if field not in fields:
+            raise Exception("Field {} is not supported by this camera".format(field))
+        return fields[field]
+
+    def __setImageCommon(self, field: str, value: str):
         return self.performRequest(
-            {
-                "method": "set",
-                "image": {"switch": {"flip_type": "center" if enable else "off"}},
-            }
+            {"method": "set", "image": {"common": {field: value}},}
         )
 
+    def getLightFrequencyMode(self) -> str:
+        return self.__getImageCommon("light_freq_mode")
+
     def setLightFrequencyMode(self, mode):
         allowed_modes = ["auto", "50", "60"]
         if mode not in allowed_modes:
             raise Exception(
                 "Light frequency mode must be one of {}".format(allowed_modes)
             )
+        return self.__setImageCommon("light_freq_mode", mode)
 
+    def getDayNightMode(self) -> str:
+        return self.__getImageCommon("inf_type")
+
+    def setDayNightMode(self, mode):
+        allowed_modes = ["off", "on", "auto"]
+        if mode not in allowed_modes:
+            raise Exception("Day night mode must be one of {}".format(allowed_modes))
+        return self.__setImageCommon("inf_type", mode)
+
+    def startManualAlarm(self):
         return self.performRequest(
-            {"method": "set", "image": {"common": {"light_freq_mode": mode}}}
+            {"method": "do", "msg_alarm": {"manual_msg_alarm": {"action": "start"}},}
+        )
+
+    def stopManualAlarm(self):
+        return self.performRequest(
+            {"method": "do", "msg_alarm": {"manual_msg_alarm": {"action": "stop"}},}
         )
 
     @staticmethod
@@ -461,7 +514,11 @@ class Tapo:
         else:
             return str(errorCode)
 
-    # test returned value
+    def getFirmwareUpdateStatus(self):
+        return self.performRequest(
+            {"method": "get", "cloud_config": {"name": "upgrade_status"}}
+        )
+
     def isUpdateAvailable(self):
         return self.performRequest(
             {
@@ -481,19 +538,66 @@ class Tapo:
             }
         )
 
-    # test returned value during update
-    def getFirmwareUpdateStatus(self):
-        return self.performRequest(
-            {"method": "get", "cloud_config": {"name": "upgrade_status"}}
-        )
-
-    # todo: test functionality
     def startFirmwareUpgrade(self):
         try:
-            started = self.performRequest(
+            self.performRequest(
                 {"method": "do", "cloud_config": {"fw_download": "null"}}
             )
-            print(started)
-            print(self.getFirmwareUpdateStatus())
         except Exception:
             raise Exception("No new firmware available.")
+
+    # Used for purposes of HomeAssistant-Tapo-Control
+    # Uses method names from https://md.depau.eu/s/r1Ys_oWoP
+    def getMost(self):
+        results = self.performRequest(
+            {
+                "method": "multipleRequest",
+                "params": {
+                    "requests": [
+                        {
+                            "method": "getDeviceInfo",
+                            "params": {"device_info": {"name": ["basic_info"]}},
+                        },
+                        {
+                            "method": "getDetectionConfig",
+                            "params": {"motion_detection": {"name": ["motion_det"]}},
+                        },
+                        {
+                            "method": "getLensMaskConfig",
+                            "params": {"lens_mask": {"name": ["lens_mask_info"]}},
+                        },
+                        {
+                            "method": "getLdc",
+                            "params": {"image": {"name": ["switch", "common"]}},
+                        },
+                        {
+                            "method": "getLastAlarmInfo",
+                            "params": {"msg_alarm": {"name": ["chn1_msg_alarm_info"]}},
+                        },
+                        {
+                            "method": "getLedStatus",
+                            "params": {"led": {"name": ["config"]}},
+                        },
+                        {
+                            "method": "getTargetTrackConfig",
+                            "params": {"target_track": {"name": ["target_track_info"]}},
+                        },
+                        {
+                            "method": "getPresetConfig",
+                            "params": {"preset": {"name": ["preset"]}},
+                        },
+                        {
+                            "method": "getFirmwareUpdateStatus",
+                            "params": {"cloud_config": {"name": "upgrade_status"}},
+                        },
+                    ]
+                },
+            }
+        )
+        returnData = {}
+        for result in results["result"]["responses"]:
+            if result["error_code"] == 0:
+                returnData[result["method"]] = result["result"]
+            else:
+                returnData[result["method"]] = False
+        return returnData
